# 备份数据库和文件
# 数据库配置
#   mysql数据库配置
#     修改mysql配置文件 /etc/mysql/conf.d/mysqldump.cnf 在client 或者 mysqldump节点下增加host/user/password，导出时可以不用配置密码。
#         [mysqldump]
#         host=localhost
#         user=root
#         password='123456'
#   postgresql数据库配置
#     在pg_hba.conf中添加用户访问认证trust（要重启），一行所有用户all，二行postgres用户
#         TYPE  DATABASE    USER     ADDRESS          METHOD
#         host    all       all    192.168.12.10/32    trust
#         host    all     postgres   192.168.12.10/32   trust
# 生成配置文件命令 rails g struggle:sql_config
# Struggle::Backup.new.do 执行备份
module Struggle
  class Backup
    # 初始化配置文件
    def initialize
      _config = YAML.load_file("#{Rails.root}/config/backup.yml")
      @@config = _config.merge(YAML.load_file("#{Rails.root}/config/database.yml")[_config["env"]])
      @@config["backup_path"] = File.expand_path(@@config["backup_path"], Rails.root)
      @@now = Time.now
      # 备份路径创建
      # 参数now，创建时间，默认当前
      @@dir = "#{@@config["backup_path"]}/#{@@now.year}/#{@@now.month}/#{@@now.day}"
      unless Dir.exist? @@dir
        FileUtils.mkdir_p @@dir
      end
    end

    # 备份数据库
    def database_backup
      _filename = "#{@@config["database"]}-#{@@now.strftime("%Y%m%d%H%M%S")}"
      filename = @@config["adapter"] == "oracle_enhanced" ? "#{_filename}.dmp" : "#{_filename}.sql"
      if @@config["adapter"] == "mysql2"
        system("mysqldump -u#{@@config["username"]} #{@@config["database"]}>#{filename}")
      elsif @@config["adapter"] == "postgresql"
        system("pg_dump -U #{@@config["username"]} -d #{@@config["database"]} -E utf8 -f #{filename} -w")
      elsif @@config["adapter"] == "oracle_enhanced"
        system("exp #{@@config["username"]}/#{@@config["password"]}@#{@@config["database"]} file=#{filename} full=y")
      end
      Struggle::ZipTool.file(filename, "#{@@dir}/#{_filename}.zip")
      File.delete filename
    end

    # 备份文件
    def file_backup
      @@config["asset_paths"].each do |assetPath|
        assetPath = "#{Rails.root}/#{assetPath}"
        ridx = assetPath.rindex("/") || 0
        Struggle::ZipTool.dir(assetPath, "#{@@dir}/#{assetPath[ridx..assetPath.length - 1]}-#{@@now.strftime("%Y%m%d%H%M%S")}.zip")
      end
    end

    # ftp上传
    def ftp_backup
      _filename = "backup-#{@@now.strftime("%Y%m%d%H%M%S")}.zip"
      ftp_config = @@config["ftp"]
      Struggle::ZipTool.dir(@@dir, _filename)
      Struggle::FtpTool.new(ftp_config["ip"], ftp_config["username"], ftp_config["password"]).put(_filename, ftp_config["backup_path"])
      File.delete _filename
    end

    # 备份
    def do
      database_backup
      file_backup
      if @@config["mode"] == "ftp"
        ftp_backup
      end
    end
  end
end