module Struggle
  class Http
    require 'net/http'
    require 'net/https'
    require 'uri'
    attr_reader :uri
    attr_reader :request

    def initialize(url)
      @uri = URI.parse(url)
      @http = Net::HTTP.new(@uri.host, @uri.port)
      if @uri.scheme == 'https'
        @http.use_ssl = true
        @http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
    end

    # get请求，params为携带的参数，header为报文头数据，两个参数都是hash类型，示例
    # http = Struggle::Http.new("http://xxx.com/xxx")
    # result = http.post({name: 'lily', age: 18}).body
    def get(params = nil, header = nil)
      @request = Net::HTTP::Get.new(@uri.request_uri, header)
      if !params.blank?
        @request.form_data = params
      end
      @http.request(@request)
    end

    # post请求，params为携带的参数为json类型，header为报文头数据为hash类型，示例：
    # http = Struggle::Http.new("http://xxx.com/xxx")
    # data = {name: 'lily', age: 18} || '{"aa":"bb"}'json类型（字符串）
    # result = http.post(data, {'Content-Type' => 'application/json'}).body
    # eval(result)                 => {status: 200, msg: 'success'}
    def post(params = nil, header = nil)
      @request = Net::HTTP::Post.new(@uri.request_uri, header)
      if !params.blank?
        if params.class == Hash
          @request.set_form_data(params)
        else
          @request.body = params
        end
      end
      @http.request(@request)
    end

    def encode(html)
      html.force_encoding("gb2312").encode("utf-8")
    end
  end
end