# = RSA工具，基于openssl，功能包括，秘钥生成，加密解密
module Struggle
  class Rsa
    # 实例化Rsa类，keyfile是秘钥文件地址（公钥或私钥）文件格式pem
    def initialize(keyfile)
      keystr = File.read(keyfile)
      if keystr
        @@rsa = OpenSSL::PKey::RSA.new(keystr)
      end
    end

    # Rsa.generate("/Users/apple/object")       =>  /Users/apple/object/private_key.pem ,public_key.pem
    # 创建2048位秘钥文件
    def self.generate(dir)
      pk = OpenSSL::PKey::RSA.generate(2048)
      File.write(dir + "/private_key.pem", pk.to_pem)
      File.write(dir + "/public_key.pem", pk.public_key.to_pem)
    end

    # 私钥加密
    def private_encrypt(value)
      Base64.encode64(@@rsa.private_encrypt(value.force_encoding("UTF-8"))) unless value.blank?
    end

    # 私钥解密
    def private_decrypt(value)
      @@rsa.private_decrypt(Base64.decode64(value.force_encoding("UTF-8"))) unless value.blank?
    end

    # 公钥加密
    def public_encrypt(value)
      Base64.encode64(@@rsa.public_encrypt(value.force_encoding("UTF-8"))) unless value.blank?
    end

    # 公钥解密
    def public_decrypt(value)
      @@rsa.public_decrypt(Base64.decode64(value.force_encoding("UTF-8"))) unless value.blank?
    end

    # 签名
    def sign(value)
      @@rsa.sign("sha1", value)
    end
  end
end